@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-# Mirrorselect 1.x
+# Mirrorselect 2.x
 # Tool for selecting Gentoo source and rsync mirrors.
 #
 # Copyright (C) 2005 Colin Kingsley <tercel@gentoo.org>
@@ -21,7 +21,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
-__revision__ = '2.0.0'
+__revision__ = '2.1.0'
 
 import math
 import os
@@ -37,7 +37,7 @@ import time
 import urllib
 import urlparse
 from optparse import IndentedHelpFormatter, OptionParser
-from mirrorselect.mirrorparser3 import MirrorParser3, MIRRORS_3_XML
+from mirrorselect.mirrorparser3 import MirrorParser3, MIRRORS_3_XML, MIRRORS_RSYNC_DATA
 import codecs
 
 class Output(object):
@@ -149,30 +149,17 @@ class Extractor(object):
 		parser = MirrorParser3()
 		self.hosts = []
 		
-		if options.rsync:
-			output.write('using hardcoded rsync mirrors\n', 2)
-			self.hosts = [
-				('rsync://rsync.gentoo.org/gentoo-portage', 'Default'),
-				('rsync://rsync.namerica.gentoo.org/gentoo-portage', 'North'
-						' America'),
-				('rsync://rsync.samerica.gentoo.org/gentoo-portage', 'South'
-						' America'),
-				('rsync://rsync.europe.gentoo.org/gentoo-portage', 'Europe'),
-				('rsync://rsync.asia.gentoo.org/gentoo-portage', 'Asia'),
-				('rsync://rsync.au.gentoo.org/gentoo-portage', 'Australia')]
-		else:
-			hosts = self.getlist(parser, list_url)
-			output.write('Extractor(): fetched mirrors.xml,'
-					' %s hosts before filtering\n' % len(hosts), 2)
+		hosts = self.getlist(parser, list_url)
+		output.write('Extractor(): fetched mirrors.xml,'
+				' %s hosts before filtering\n' % len(hosts), 2)
 
+		if not options.rsync:
 			if options.ftp:
 				hosts = self.restrict_protocall('ftp', hosts)
 			if options.http:
 				hosts = self.restrict_protocall('http', hosts)
 
-			for host in hosts:
-				if not host[0].startswith('rsync'):
-					self.hosts.append(host)
+		self.hosts = hosts
 
 	def restrict_protocall(self, prot, hosts):
 		"""
@@ -208,18 +195,13 @@ class Extractor(object):
 		except EnvironmentError:
 			pass
 
-		def fix_tuple(_tuple):
-			url, (name, country) = _tuple
-			return (url, codecs.encode(name, 'utf8'), codecs.encode(country, 'utf8'))
-
-		tuples = [fix_tuple(t) for t in parser.tuples()]
-		if len(tuples) == 0:
+		if len(parser.tuples()) == 0:
 			output.print_err('Could not get mirror list. Check your internet'
 					' connection.')
 
-		output.write(' Got %d mirrors.\n' % len(tuples))
+		output.write(' Got %d mirrors.\n' % len(parser.tuples()))
 
-		return tuples
+		return parser.tuples()
 
 
 class Shallow(object):
@@ -598,40 +580,46 @@ class Deep(object):
 class Interactive(object):
 	"""Handles interactive host selection."""
 
-	def __init__(self, hosts, rsync):
+	def __init__(self, hosts, options):
 		self.urls = []
 
-		self.interactive(hosts, rsync)
+		self.interactive(hosts, options)
 		output.write('Interactive.interactive(): self.urls = %s\n' % self.urls, 2)
 
 		if len(self.urls[0]) == 0:
 			sys.exit(1)
 	
 	
-	def interactive(self, hosts, rsync):
+	def interactive(self, hosts, options):
 		"""
 		Some sort of interactive menu thingy.
 		"""
-		if rsync:
+		if options.rsync:
 			dialog = 'dialog --stdout --title "Gentoo RSYNC Mirrors"'\
 				' --radiolist "Please select your desired mirror:" 20 110 14'
 		else:
 			dialog = 'dialog --separate-output --stdout --title'\
 				' "Gentoo Download Mirrors" --checklist "Please'\
-				' select your desired mirrors:\n* = supports ipv6" 20 110 14'
+				' select your desired mirrors:'
+			if not options.ipv4 and not options.ipv6:
+				dialog += '\n* = supports ipv6'
 
-		if rsync:
-			dialog += ' ' + ' '.join(['"%s" "%s" "OFF"' % host for host in hosts])
-		else:
-			dialog += ' ' + ' '.join(['"%s" "%s: %s" "OFF"' % \
-				(url, country, name) for (url, name, country) in \
-				sorted(hosts, key = lambda x: x[2].lower())])
-		
-		mirror_fd = os.popen('%s' % dialog)
+			dialog += '" 20 110 14'
+
+		for (url, args) in sorted(hosts, key = lambda x: (x[1]['country'].lower(), x[1]['name'].lower()) ):
+			marker = ""
+			if (not options.ipv6 and not options.ipv4) and args['ipv6'] == 'y':
+				marker = "* "
+			if options.ipv6 and ( args['ipv6'] == 'n' ): continue
+			if options.ipv4 and ( args['ipv4'] == 'n' ): continue
+
+			dialog += ' ' + '"%s" "%s%s: %s" "OFF"' % ( url, marker, args['country'], args['name'] )
+
+		mirror_fd = os.popen('%s' % codecs.encode(dialog, 'utf8'))
 		mirrors = mirror_fd.read()
 		mirror_fd.close()
 		
-		self.urls = mirrors.split('\n')
+		self.urls = mirrors.rstrip().split('\n')
 
 
 def _have_bin(name):
@@ -843,7 +831,7 @@ def parse_args(argv):
 	if options.ipv4 and options.ipv6:
 		output.print_err('Choose at most one of --ipv4 and --ipv6')
 
-	if options.ipv6 and not socket.has_ipv6:
+	if (options.ipv6 and not socket.has_ipv6) and not options.interactive:
 		options.ipv6 = False
 		output.print_err('The --ipv6 option requires python ipv6 support')
 
@@ -882,10 +870,13 @@ def main(argv):
 	output.verbosity = options.verbosity
 
 	fsmirrors = get_filesystem_mirrors(options.output, config_path, options.rsync)
-	hosts = Extractor(MIRRORS_3_XML, options).hosts
+	if options.rsync:
+		hosts = Extractor(MIRRORS_RSYNC_DATA, options).hosts
+	else:
+		hosts = Extractor(MIRRORS_3_XML, options).hosts
 
 	if options.interactive:
-		selector = Interactive(hosts, options.rsync)
+		selector = Interactive(hosts, options)
 	elif options.deep:
 		selector = Deep(hosts, options)
 	else:
